from owl_dev.database import DeclarativeBase
from sqlalchemy import Column, DateTime, Integer, Sequence, Unicode


class Info(DeclarativeBase):
    __tablename__ = "info"
    id = Column(Integer, Sequence("info_id_seq"), primary_key=True)
    config = Column(Unicode, nullable=False)
    env = Column(Unicode, nullable=False)
    python = Column(Unicode, nullable=False)



import functools
import json
import os
import shutil
import sys
from contextlib import closing, suppress
from pathlib import Path

from owl_dev import database as db

__author__ = "Eduardo Gonzalez Solares"
__email__ = "eglez@ast.cam.ac.uk"
__version__ = "0.2.1"


OWL_SUCCESS = "OWL_SUCCESS"
OWL_ERROR = "OWL_ERROR"

SQLITEDB = "sqlite.db"


class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Path):
            return obj.as_posix()
        return json.JSONEncoder.default(self, obj)


def setup_output(output_dir, clean=False):
    output_dir.mkdir(parents=True, exist_ok=True)
    if clean:
        shutil.rmtree(output_dir)

    with suppress(Exception):
        (output_dir / OWL_SUCCESS).unlink()

    with suppress(Exception):
        (output_dir / OWL_ERROR).unlink()

    with suppress(Exception):
        (output_dir / SQLITEDB).unlink()

    with open(f"{output_dir}/env.yaml", "w") as fh:
        fh.write(json.dumps(dict(os.environ)))


def pipeline(callable=None):
    def decorator(function):
        @functools.wraps(function)
        def wrapper(*args, **kwargs):
            try:
                pdef = wrapper.config
            except Exception:
                pdef = {}

            if callable is not None:
                pre = callable(**kwargs)
                output_dir = pre.get("output_dir")
                clean_output = pre.get("clean_output", False)
            else:
                for k in ["output", "output_dir"]:
                    output_dir = kwargs.get(k, None)
                    if output_dir is not None:
                        break
                clean_output = kwargs.get("clean_output", False)

            if output_dir is not None:
                setup_output(output_dir, clean=clean_output)

                # db.init_database(f"sqlite:///{output_dir}/{SQLITEDB}")
                with open(f"{output_dir}/config.yaml", "w") as fh:
                    fh.write(json.dumps(pdef))

            # else:
            #     db.init_database("sqlite:///:memory:")

            # with closing(db.DBSession()) as session:
            #     info = db.Info(
            #         config=JSONEncoder().encode(pdef),
            #         env=JSONEncoder().encode(dict(os.environ)),
            #         python=sys.version,
            #     )
            #     session.add(info)
            #     session.commit()

            try:
                success = False
                result = function(*args, **kwargs)
                success = True
            finally:
                if output_dir is not None:
                    if success:
                        (output_dir / OWL_SUCCESS).touch()
                    else:
                        (output_dir / OWL_ERROR).touch()

            return result

        return wrapper

    if callable is not None:
        return decorator(callable)

    return decorator




from setuptools import find_packages, setup

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("requirements.txt") as requirements_file:
    requirements = requirements_file.read()

setup_requirements = ["pytest-runner", "flake8"]

test_requirements = ["coverage", "pytest", "pytest-cov", "pytest-mock"]

setup(
    author="Eduardo Gonzalez Solares",
    author_email="eglez@ast.cam.ac.uk",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.6",
    ],
    description="Pipeline Framework",
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme,
    include_package_data=True,
    keywords="owl, pipeline",
    name="owl-pipeline-develop",
    packages=find_packages(include=["owl_dev*"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/IMAXT/owl-pipeline-develop",
    download_url="https://github.com/IMAXT/owl-pipeline-develop/archive/v0.1.20.tar.gz",
    version="0.2.1",
    zip_safe=False,
    python_requires=">=3",
)



from typing import Any

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import scoped_session, sessionmaker

maker = sessionmaker(autoflush=True, autocommit=False)
DBSession = scoped_session(maker)

DeclarativeBase: Any = declarative_base()
# for another schema
# declarative_base(metadata=MetaData(schema='public'))
metadata = DeclarativeBase.metadata


def init_model(engine):  # pragma: nocover
    """Call me before using any of the tables or classes in the model."""
    DBSession.configure(bind=engine)


def init_database(url, echo=False):  # pragma: nocover
    engine = create_engine(url, echo=echo)
    init_model(engine)
    metadata.create_all(engine)


# Import your model modules here.
from owl_dev.database.model import Info  # noqa  # isort:skip
